import {
  csv,
  select,
  scaleLinear,
  axisBottom,
  axisLeft,
  symbolCircle,
  symbolSquare,
  symbolStar,
  symbol,
} from 'd3';
import { scatterPlot } from './scatterPlot';

const csvUrl =
  'https://gist.githubusercontent.com/sagam929/e18cebb98ecf2dce1f1d178f26ff4f34/raw/e9eccb590e0156762a6222af07f91fa245d851fa/University_Ranks.csv';

// Define symbol mapping
const symbolMapping = {
  'Research Environment': d3.symbol().type(symbolStar),
  'Students to Staff Ratio': d3.symbol().type(symbolSquare),
  'Research Quality': d3.symbol().type(symbolCircle),
  'Industry Impact': d3.symbol().type(symbolSquare),
};

const types = Object.keys(symbolMapping);

const svg = select('body')
  .append('svg')
  .attr('width', window.innerWidth)
  .attr('height', window.innerHeight);

const parseRow = (d) => {
  // Ensure numeric conversion for relevant fields
  d['Students to Staff Ratio'] =
    +d['Students to Staff Ratio'];
  d['Research Environment'] = +d['Research Environment'];
  d['Research Quality'] = +d['Research Quality'];
  d['Industry Impact'] = +d['Industry Impact'];
  d['Year'] = +d['Year'];
  return d;
};

const main = async () => {
  const data = await csv(csvUrl, parseRow);
  const countries = data.map((d) => d.Country); // Extract country names
  const xScale = scaleLinear()
    .domain([0, countries.length - 1]) // Scale for x-axis representing countries
    .range([100, window.innerWidth - 100]); // Adjust the range as per your requirement

  let index = 0; // To cycle through cancer types
  const updatePlot = () => {
    const cancerType = types[index % types.length];
    const yScale = scaleLinear()
      .domain([0, d3.max(data, (d) => d[cancerType])])
      .range([window.innerHeight - 100, 100]); // Adjust the range as per your requirement

    const updatedData = data.map((d, i) => ({
      x: i, // Map index directly to x, assuming data is ordered by country
      y: d[cancerType],
      country: d.Country, // Include country name for tooltip or identification
    }));

    const xAxis = axisBottom(xScale).tickFormat(
      (i) => countries[i],
    ); // Format ticks with country names
    const yAxis = axisLeft(yScale);

    svg.selectAll('*').remove(); // Clear previous content
    svg
      .append('g')
      .attr(
        'transform',
        `translate(0, ${window.innerHeight - 100})`,
      )
      .call(xAxis)
      .selectAll('text')
      .attr('transform', 'translate(-10,0)rotate(-90)')
      .style('text-anchor', 'end');
    svg
      .append('g')
      .attr('transform', 'translate(100, 0)')
      .call(yAxis);

    scatterPlot()
      .width(window.innerWidth)
      .height(window.innerHeight)
      .data(updatedData)
      .xValue((d) => xScale(d.x))
      .yValue((d) => yScale(d.y))
      .margin({
        top: 100,
        right: 20,
        bottom: 100,
        left: 150,
      })
      .symbolType(symbolMapping[cancerType])(svg);
    index++;
    setTimeout(updatePlot, 2000); // Update every 2 seconds
  };
  updatePlot();
};

main();
